#!/usr/bin/env python3
"""
Sub-Package GUI.TOOLKITS.KDE4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008, 2009 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains the KDE toolkit modules.
"""
