#!/usr/bin/env python3
"""
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package provides the base GUI objects for
creating a "standard" application framework --

* Menu and menu item management
* Toolbar and tool button management
* Actions automatically linked to menu and toolbar items
* Application main window management

These 'base' objects serve as mixin classes for the GUI
objects in each toolkit.
"""
