#!/usr/bin/env python3
"""
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package provides helper/mixin classes for use by
GUI widgets regardless of the GUI toolkit being used. Each
'Base' class defines the API for its widget type.
"""
